function path = fixedHorizonSDP(load_disc,trans_matrix,p_pv,spotprice,...
    utilityprice,structure,p_bat_max,eff_ch,eff_di,C_bat,SOC_max,...
    SOC_min,N_soc,SOC_initial,SOC_final,daynum)

% Assuming p_pv and prices has the same dimensions, here 1xN_horizon

% load_disc contains the discrete load states, and has the dimensions
% (N_states, N_periods).

% trans_matrix has the dimensions (N_states, N_periods, N_states), and 
% gives the probability of the transition from the 1st dim state to the 
% 3rd dim state given that you are in the period in the 2nd dim.

%% Setting key parameters
N_periods = size(load_disc,2);  % Number of time steps in the horizon.
N_disc = size(load_disc,1);     % Number of discrete loads.

% Maximum changes in SOC state
soc_max_change_ch = floor(p_bat_max*eff_ch*N_soc/(C_bat));
soc_max_change_di = floor(p_bat_max*N_soc/(C_bat*eff_di));

if SOC_initial == SOC_min
    N_soc_initial = 1;
else
    % Yields which discrete step the initial SOC belongs to
    N_soc_initial = floor(N_soc*(SOC_initial-SOC_min)/(SOC_max-SOC_min));
end

if SOC_final == SOC_min
    N_soc_final = 1;
else
    % Yields which discrete step the final SOC belongs to
    N_soc_final = floor(N_soc*(SOC_final-SOC_min)/(SOC_max-SOC_min));
end

%% Initialising arrays
cost_to_go = inf(N_soc,N_periods,N_disc);
transitioncost = inf(N_soc,N_disc);     
path = zeros(N_soc,N_periods,N_disc);  

% Calculating all legal SOC states. SOC_legal is a binary matrix, 1 if the 
% given SOC is legal for the given period. Else 0.
SOC_legal = socLegalCalc(N_soc,N_periods,N_soc_initial,N_soc_final,...
    soc_max_change_ch,soc_max_change_di);

% Fixed final SOC = 0
cost_to_go(:,N_periods,:) = 0;  
transitioncost(N_soc_final,:) = 0;

%% Calculations
for t = (N_periods-1):-1:1 % Period iterated recursively
        
    %% Calculate costs
    for soc_t = 1:N_soc % Iterates over all legal soc's in period t
        
        if SOC_legal(soc_t,t) == 1

            soc_max_tp1 = min(N_soc,soc_t+soc_max_change_ch);
            soc_min_tp1 = max(1,soc_t-soc_max_change_di);
            
            for load_t = 1:N_disc 
                % Iterates over all possible load scenarios in period t.
            
                for soc_tp1 = soc_min_tp1:soc_max_tp1 
                    % Iterates over all legal SOC's in period t+1 which can
                    % be reached from soc_t.

                    if SOC_legal(soc_tp1,t+1) == 1

                        SOC_change = (soc_tp1-soc_t)/N_soc;
                        p_bat = SOC_change*C_bat;

                        if p_bat > 0
                            p_bat = p_bat/eff_ch;  % eff_ch < 1
                        else
                            p_bat = p_bat*eff_di;  % eff_di < 1
                        end

                        p_grid = load_disc(load_t,t) + p_bat - p_pv(t);

                        if p_grid > 0
                            switch structure
                                case 1
                                    % Only spotprice
                                    transitioncost(soc_tp1,load_t) = ...
                                        spotprice(t)*p_grid;

                                case 2
                                    % Flat rate                       
                                    transitioncost(soc_tp1,load_t) = ...
                                        (spotprice(t) + ...
                                        utilityprice)*p_grid;

                                case 3
                                    % Timely

                                    transitioncost(soc_tp1,load_t) = ...
                                        (spotprice(t) + ...
                                        utilityprice(daynum,t))*p_grid;

                                case 4
                                    % P-based

                                    utilityprice_temp = utilityprice*...
                                        p_grid;
                                    transitioncost(soc_tp1,load_t) = ...
                                        (spotprice(t) + ...
                                        utilityprice_temp)*p_grid;

                            end % switch structure
                        else
                            transitioncost(soc_tp1,load_t) = ...
                                spotprice(t)*p_grid;
                        end % if p_grid > 0
                        
                        for load_tp1 = 1:N_disc
                            % Add expected cost_to_go using the transition
                            % probabilities in trans_matrix
                            if transitioncost(soc_tp1,load_t) ~= inf
                                transitioncost(soc_tp1,load_t) = ...
                                    transitioncost(soc_tp1,load_t) + ...
                                    cost_to_go(soc_tp1,t+1,load_tp1)*...
                                    trans_matrix(load_t,t,load_tp1);
                            end
                        end
                        
                    end % if SOC_legal(t+1,soc_tp1) == 1
                end % for soc_tp1
                
                [min_cost, min_soc] = min(transitioncost(:,load_t));
            
                cost_to_go(soc_t,t,load_t) = min_cost;

                path(soc_t,t,load_t) = min_soc;
                
            end % for load_t = 1:N_states
            
        end % SOC_legal(t,soc_t) == 1
        
        transitioncost = inf(N_soc,N_disc);   % Reset of transitioncosts
        
    end % for soc_t
    
end % for t

end % function